@@ -295,8 +295,8 @@ def lines(start=None, end=None, reverse=False, max_steps=None):
     :param max_steps: Maximum number of steps to iterate.
     :yields: instructions addresses
     """
-    max_ea = idaapi.cvar.inf.maxEA
-    min_ea = idaapi.cvar.inf.minEA
+    max_ea = idaapi.cvar.inf.max_ea
+    min_ea = idaapi.cvar.inf.min_ea
 
     # Normalize start and end addresses.
     if reverse:
@@ -370,7 +370,7 @@ def is_x86_64():
     :return bool: True if an Intel x86 or x64, False otherwise
     """
     info = idaapi.get_inf_structure()
-    return info.procName == "metapc"
+    return info.procname == "metapc"
 
 
 def is_ARM():
@@ -380,4 +380,4 @@ def is_ARM():
     :return bool: True if an ARM processor, False otherwise
     """
     info = idaapi.get_inf_structure()
-    return info.procName == "ARM"
+    return info.procname == "ARM"
